// EDock (C)opyright 2007 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/application.h>
#include <util/event.h>
#include <gui/layoutview.h>
#include <gui/separator.h>
#include <gui/imagebutton.h>
#include <storage/registrar.h>
#include <gui/desktop.h>
#include "windowview.h"
#include "dockview.h"
#include "appwindow.h"
#include "common.h"

using namespace os;

///////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////

class AppWindow :: _Private
{
public:
	_Private()
	{
		// Get the registrar manager
		m_Manager = NULL;
		try
		{
			m_Manager = RegistrarManager::Get();
		} catch( ... )
		{
		}
	
		// Create desktop handle
		m_Desktop = new os::Desktop();
	
	}


	// Returns the frame of the dock
	Rect _GetDockFrame()
	{
		if( m_Desktop == NULL )
			return( Rect( 0, 0, 0, 0) );
	
		Rect cFrame;
	
		cFrame = Rect( 0, m_Desktop->GetResolution().y - 31, m_Desktop->GetResolution().x - 1, m_Desktop->GetResolution().y - 1 );
		
		return( cFrame );
	}

	enum AppWindowMessage { MSG_SHOW_DESKTOP, MSG_WINDOW_EV };
	
	// Global pointers
	Desktop* m_Desktop;
	RegistrarManager* m_Manager;

	// Widgets
	ImageButton* m_ShowDesktop;
	Separator* m_Separator;
	WindowView* m_WindowView;

	// Events
	Event* m_WindowEv;
};

///////////////////////////////////////////////////////////////////////////////
//
// T H E   C L A S S
//
///////////////////////////////////////////////////////////////////////////////
AppWindow :: AppWindow() : Window( os::Rect(), "edock_win", "EDock", WND_NO_BORDER, ALL_DESKTOPS )
{	
	// Create the private class
	m = new _Private();

	// Create widgets
	LayoutNode*	root_node = new HLayoutNode( "pcRootNode" );
	DockView* root_view = new DockView( GetBounds(), "pcRootView", root_node);
	AddChild( root_view );
	
	// Create Show Desktop button
	m->m_ShowDesktop = new ImageButton( Rect(), "desktop_button", "", new Message( _Private::MSG_SHOW_DESKTOP ), LoadImage( "desktop.png" ) );
	m->m_Separator = new Separator( Rect(), "" );
	m->m_WindowView = new WindowView();

	// Add widgets to the layout system
	root_node->AddChild( m->m_ShowDesktop, 0.0f );
	root_node->AddChild( m->m_Separator, 0.0f );
	root_node->AddChild( m->m_WindowView, 1.0f );
	

 	// Resize ourself
	SetFrame( m->_GetDockFrame() );

	root_view->InvalidateLayout();

	// Bind to window event
	m->m_WindowEv = new Event();
	m->m_WindowEv->SetToRemote( "os/Window/", -1 );
	m->m_WindowEv->SetMonitorEnabled( true, this, _Private::MSG_WINDOW_EV );
}

AppWindow :: ~AppWindow()
{
	delete m;
}

void AppWindow :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{	
		case _Private::MSG_SHOW_DESKTOP:
			m->m_Desktop->MinimizeAll();
			break;
		case _Private::MSG_WINDOW_EV:
			m->m_WindowView->UpdateWindowView( pcMessage );
			break;
		default:
			Window::HandleMessage( pcMessage );
			break;
	}
}

bool AppWindow :: OkToQuit( void )
{
	Application::GetInstance()->PostMessage( M_QUIT );
	return true;	
}

void AppWindow :: DesktopActivated( int nDesktop, bool bActive )
{
	/* Update desktop parameters */
	delete( m->m_Desktop );
	m->m_Desktop = new os::Desktop();
	
	/* Resize ourself */
	SetFrame( m->_GetDockFrame() );	

	m->m_WindowView->UpdateWindowView();
}

// Returns the current desktop
uint32 AppWindow :: GetDesktop() 
{ 
	return m->m_Desktop->GetDesktop();
}
